'
' channel.exe passes channel change requests to WinLIRC via its network interface.
'
' Usage: channel [channel number]
'   Running channel without parameters starts the configuration program.
' 
' Original source code from http://computersmithery.com/Channel.aspx
' Logging and network timeouts added by Puckdropper, http://puckdroppersplace.us/computer/
'
' Brett Smith wrote the original source code, and has graciously permitted me to post it as well as the new code for Logging and network timeouts.
'

Option Strict On
Option Explicit On
Imports System.IO

Module WinLIRC
    Public Timeout As Integer
    Public Log As Boolean
    Public strRemote As String
    Public intRepeat As Integer
    Public intPadding As Integer
    Public strConfirmCMD As String
    Public intWaitTime As Integer
    Public strServerIP As String
    Public intServerPort As Integer

    Public Function ChangeChannelNew(ByVal Channel As Integer) As Boolean
        Dim NoErrors As Boolean = True

        ' Returns true on no error, false on anything else
        Write("Changing to channel " & Channel.ToString)

        Dim tcpc As System.Net.Sockets.TcpClient = New System.Net.Sockets.TcpClient()
        'attempt to connect to the winlirc server
        Try
            tcpc.Connect(strServerIP, intServerPort)
        Catch socketEx As System.Net.Sockets.SocketException
            Write("[Changing to channel] Error: Could not connect to the WinLIRC server")
            NoErrors = False
        End Try

        'Prep for sending the winlirc command 
        Dim nwstream As System.Net.Sockets.NetworkStream = tcpc.GetStream()
        Dim strResponse As String = ""

        Try
            Dim strChannel As String = Channel.ToString
            Dim strCMD As String

            'pad the channel if required
            If intPadding > 0 Then
                Do Until Not strChannel.Length < intPadding
                    strChannel = "0" & strChannel
                Loop
            End If
            'loop through the digits in the channel number
            For Each chrDigit As Char In strChannel
                strCMD = "SEND_ONCE " & strRemote & " " & chrDigit & " " & intRepeat.ToString
                doChangeChannel(strCMD, nwstream, strResponse)
                System.Threading.Thread.Sleep(intWaitTime)
            Next
            ' Send Confirmation Code if needed
            If strConfirmCMD <> "" Then
                strCMD = "SEND_ONCE " & strRemote & " " & strConfirmCMD & " " & intRepeat.ToString
                doChangeChannel(strCMD, nwstream, strResponse)
            End If
        Catch ex As Exception
            Write("Error:  A serious error has occurred: " & ex.Message.ToString)
            NoErrors = False
        Finally
            tcpc.Close()
            If Not (nwstream Is Nothing) Then
                nwstream = Nothing
            End If
            If Not (tcpc Is Nothing) Then
                tcpc = Nothing
            End If
        End Try
        Return NoErrors
    End Function

    Private Function WriteToStream(ByRef nwstream As System.Net.Sockets.NetworkStream, ByVal strLine As String) As Boolean
        Dim strString2Send As String = strLine & Chr(13) & Chr(10)
        Dim arr2Send As Byte() = System.Text.Encoding.ASCII.GetBytes(strString2Send.ToCharArray())
        Try
            ' Network time outs added by Puckdropper
            nwstream.WriteTimeout = TIMEOUT
            nwstream.Write(arr2Send, 0, arr2Send.Length)
        Catch ex As Exception
            Write("Could not write to stream:" & ex.Message.ToString)
            Return False
        End Try
        Return True
    End Function

    Private Function ReadFromStream(ByRef nwstream As System.Net.Sockets.NetworkStream, ByRef strMessage As String) As Boolean
        Dim readBuffer(255) As Byte
        If (nwstream.CanRead()) Then
            'Network timeout added by Puckdropper
            nwstream.ReadTimeout = TIMEOUT
            Dim nLength As Integer = nwstream.Read(readBuffer, 0, readBuffer.Length)
            strMessage = System.Text.Encoding.ASCII.GetString(readBuffer, 0, nLength)
            Write("WinLIRC Response: ")
            Write(strMessage)
            Return (3 <= readBuffer(0))

        Else
            Return False
        End If
    End Function

    Public Sub Write(ByVal LogText As String)
        REM Logging Added by Puckdropper
        REM puckdropper at fastmail dot fm
        Console.WriteLine(LogText)
        If (Log) Then
            Dim Writer As StreamWriter = File.AppendText("channel.Log")
            Writer.WriteLine("(" & Now.ToString & ") " & LogText)
            Writer.Close()
        End If
    End Sub

    Private Sub doChangeChannel(ByVal strCMD As String, ByVal nwstream As System.Net.Sockets.NetworkStream, ByVal strResponse As String)
        Write("Sending command:" & strCMD)

        WriteToStream(nwstream, strCMD)
        If ReadFromStream(nwstream, strResponse) = False Then
            Write("Error: No response from received from the WinLIRC server.")
        End If
    End Sub
End Module
